@@ -33,7 +33,7 @@ def set_reset_less(field):
             s.reset_less = True
 
 class EndpointDescription:
-    def __init__(self, payload_layout, param_layout=[]):
+    def __init__(self, payload_layout=[], param_layout=[]):
         self.payload_layout = payload_layout
         self.param_layout   = param_layout
 
@@ -59,7 +59,7 @@ class EndpointDescription:
 
 
 class Endpoint(Record):
-    def __init__(self, description_or_layout, name=None, **kwargs):
+    def __init__(self, description_or_layout=[], name=None, **kwargs):
         if isinstance(description_or_layout, EndpointDescription):
             self.description = description_or_layout
         else:
@@ -228,13 +228,31 @@ class SyncFIFO(_FIFOWrapper):
 
 
 class AsyncFIFO(_FIFOWrapper):
-    def __init__(self, layout, depth=4, buffered=False):
+    def __init__(self, layout, depth=None, buffered=False):
+        depth = 4 if depth is None else depth
         assert depth >= 4
         _FIFOWrapper.__init__(self,
             fifo_class = fifo.AsyncFIFOBuffered if buffered else fifo.AsyncFIFO,
             layout     = layout,
             depth      = depth)
 
+# ClockDomainCrossing ------------------------------------------------------------------------------
+
+class ClockDomainCrossing(Module):
+    def __init__(self, layout, cd_from="sys", cd_to="sys", depth=None):
+        self.sink   = Endpoint(layout)
+        self.source = Endpoint(layout)
+        # # #
+
+        if cd_from == cd_to:
+            self.comb += self.sink.connect(self.source)
+        else:
+            cdc = AsyncFIFO(layout, depth)
+            cdc = ClockDomainsRenamer({"write": cd_from, "read": cd_to})(cdc)
+            self.submodules += cdc
+            self.comb += self.sink.connect(cdc.sink)
+            self.comb += cdc.source.connect(self.source)
+
 # Mux/Demux ----------------------------------------------------------------------------------------
 
 class Multiplexer(Module):
